from sgqlc.types import Arg, Enum, Field, Input, Int, String, Type, list_of, non_null
from sgqlc.types.relay import Node


class Facility(Type, Node):
    id = Field(non_null(Int), graphql_name="id")
    name = Field(non_null(String), graphql_name="name")
    slug = Field(non_null(String), graphql_name="slug")
    address = Field(String, graphql_name="address")
    city = Field(String, graphql_name="city")
    state = Field(String, graphql_name="state")
    zip = Field(String, graphql_name="zip")
    timezone_name = Field(String, graphql_name="timezoneName")


class FacilitiesConnection(Type, Node):
    nodes = list_of(Facility)


class FacilityInput(Input):
    name = non_null(str)
    slug = non_null(str)
    timezone_name = non_null(str)


class CreateFacilityInput(Input):
    facility = Field(non_null("FacilityInput"), graphql_name="facility")


class CreateFacilityPayload(Type):
    facility = Field("Facility", graphql_name="facility")


class MetricLabel(Type):
    source_id = non_null(str)
    label = non_null(str)


class MetricDataAggregationMethod(Enum):
    __choices__ = ("AVG", "COUNT", "FIRST", "LAST", "MAX", "MEDIAN", "MIN", "SUM")


class MetricDataOrderBy(Enum):
    __choices__ = ("TIME_ASC", "TIME_DESC")


class MetricData(Type, Node):
    time = non_null(str)
    source_id = Field(non_null(str), graphql_name="sourceId")
    label = str
    data = str
    metadata = str


class MetricDataConnection(Type, Node):
    nodes = list_of(MetricData)


class Query(Type):
    facilities = Field(FacilitiesConnection)
    metric_labels = Field(non_null(list_of(non_null(MetricLabel))), graphql_name="metricLabels")
    metric_data = Field(
        MetricDataConnection,
        graphql_name="metricData",
        args={
            "source_id": Arg(non_null(str), graphql_name="sourceId", default=None),
            "label": Arg(non_null(str), graphql_name="label", default=None),
        },
    )


class Mutation(Type):
    create_facility = Field(
        CreateFacilityPayload,
        graphql_name="createFacility",
        args={"input": Arg(non_null(CreateFacilityInput))},
    )



import click

from contxt.cli.clients import Clients
from contxt.cli.utils import Serializer


@click.group()
def metrics() -> None:
    """Metrics."""


@metrics.command()
@click.pass_obj
def labels(clients: Clients) -> None:
    """Get Metric Labels"""
    labels = clients.nionic.get_metric_labels()
    print(Serializer.to_table(labels))


@metrics.command()
@click.option(
    "--source-id",
    required=True,
    help="Organization ID, defaults to first value in token if not specified",
)
@click.option(
    "--label", required=True, help="Organization ID, defaults to first value in token if not specified"
)
@click.pass_obj
def data(clients: Clients, source_id: str, label: str) -> None:
    """Get facilities"""
    facilities = clients.nionic.get_metric_data(label, source_id)
    print(Serializer.to_table(facilities))



from datetime import date, datetime, timedelta, timezone
from typing import Iterable, List, Optional

from ..auth import Auth
from ..models.ems import Facility, MainService, ResourceType, UtilityContract, UtilitySpend, UtilityUsage
from ..utils.orgs import get_slug_or_org_id
from .api import ApiEnvironment, ConfiguredApi
from .pagination import PagedRecords


class EmsService(ConfiguredApi):
    """EMS API client"""

    _envs = (
        ApiEnvironment(
            name="production",
            base_url="https://{tenant}.api.ndustrial.io/rest/ems/v1",
            client_id="e2IT0Zm9RgGlDBkLa2ruEcN9Iop6dJAS",
        ),
        ApiEnvironment(
            name="staging",
            base_url="https://{tenant}.api.staging.ndustrial.io/rest/ems/v1",
            client_id="vMV67yaRFgjBB1JFbT3vXBOlohFdG1I4",
        ),
    )

    def __init__(self, auth: Auth, org_id: str, env: str = "production", **kwargs) -> None:
        super().__init__(env=env, auth=auth, **kwargs)
        self.org_id = org_id
        tenant = get_slug_or_org_id(org_id)
        self.base_url = self.base_url.format(tenant=tenant)

    def get_facility(self, id: int) -> Facility:
        return Facility.from_api(self.get(f"{self.org_id}/facilities/{id}"))

    def get_main_services(
        self, facility_id: int, resource_type: Optional[ResourceType] = None
    ) -> List[MainService]:
        facility = self.get_facility(facility_id)
        main_services = facility.main_services

        # Manually filter on resource type
        if resource_type:
            main_services = [s for s in main_services if s.resource_type == resource_type]
        return main_services

    def get_monthly_utility_spend(
        self,
        facility_id: int,
        resource_type: ResourceType = ResourceType.ELECTRIC,
        start_date: date = date.today() - timedelta(days=3600),
        end_date: date = date.today(),
        pro_forma: bool = False,
        exclude_account_charges: bool = False,
    ) -> UtilitySpend:
        """Get monthly utility spend for facility `facility_id` and resource
        `resource_type` (for now, this must be "electric" but will be expanded).
        Note `start_date` defaults to 10 years ago (the API's maximum limit) and
        `end_date` defaults to today."""
        resp = self.get(
            f"{self.org_id}/facilities/{facility_id}/utility/spend/monthly",
            params={
                "type": resource_type.value,
                "date_start": start_date.strftime("%Y-%m"),
                "date_end": end_date.strftime("%Y-%m"),
                "pro_forma": pro_forma,
                "exclude_account_charges": exclude_account_charges,
            },
        )
        return UtilitySpend.from_api(resp)

    def get_monthly_utility_usage(
        self,
        facility_id: int,
        resource_type: ResourceType = ResourceType.ELECTRIC,
        start_date: date = date.today() - timedelta(days=3600),
        end_date: date = date.today(),
        pro_forma: bool = False,
    ) -> UtilityUsage:
        """Get monthly utility usage for facility `facility_id` and resource
        `resource_type` (for now, this must be "electric" but will be expanded).
        Note `start_date` defaults to 10 years ago (the API's maximum limit) and
        `end_date` defaults to today."""
        resp = self.get(
            f"{self.org_id}/facilities/{facility_id}/utility/usage/monthly",
            params={
                "type": resource_type.value,
                "date_start": start_date.strftime("%Y-%m"),
                "date_end": end_date.strftime("%Y-%m"),
                "pro_forma": pro_forma,
            },
        )
        return UtilityUsage.from_api(resp)

    def get_usage(
        self,
        facility_id: int,
        interval: str,
        resource_type: ResourceType = ResourceType.ELECTRIC,
        start: datetime = datetime.now(timezone.utc) - timedelta(days=365),
        end: datetime = datetime.now(timezone.utc),
    ) -> UtilityUsage:
        """Get utility usage for facility `facility_id` and resource `resource_type`"""
        resp = self.get(
            f"{self.org_id}/facilities/{facility_id}/usage/{interval}",
            params={
                "type": resource_type.value,
                "time_start": int(start.timestamp()),
                "time_end": int(end.timestamp()),
            },
        )
        return UtilityUsage.from_api(resp)

    def get_utility_contracts_for_facility(self, facility_id: int) -> Iterable[UtilityContract]:
        return PagedRecords(
            api=self,
            url=f"{self.org_id}/facilities/{facility_id}/contracts",
            record_parser=UtilityContract.from_api,
        )



from typing import List

import click

from contxt.cli.clients import Clients
from contxt.cli.utils import fields_option, print_table, sort_option
from contxt.models.contxt import EdgeNode


@click.group("edge-nodes")
def edge_nodes() -> None:
    """Edge Nodes."""


@edge_nodes.command()
@click.argument("org_id")
@click.argument("project_id")
@fields_option(default="id, name, stack_id, description", obj=EdgeNode)
@sort_option(default="id")
@click.pass_obj
def get(clients: Clients, org_id: str, project_id: int, fields: List[str], sort: str) -> None:
    """Get edge nodes for a project"""
    items = clients.contxt.get_edge_nodes(organization_id=org_id, project_id=project_id)
    print_table(items=items, keys=fields, sort_by=sort)



from typing import List

from sgqlc.endpoint.requests import RequestsEndpoint
from sgqlc.operation import Operation

from contxt.models.facilities import Facility, MetricData, MetricLabel, Mutation, Query
from contxt.services.api import ApiEnvironment, BaseGraphService

from ..auth import Auth
from ..utils.orgs import get_slug_or_org_id


class NionicService(BaseGraphService):
    """Nionic API client"""

    _envs = (
        ApiEnvironment(
            name="production",
            base_url="https://{tenant}.api.ndustrial.io",
            client_id="vtiZlMRo4apDvThTRiH7kLifQXWUdi9j",
        ),
        ApiEnvironment(
            name="staging",
            base_url="https://{tenant}.api.staging.ndustrial.io",
            client_id="vhGxildn8hRRWZj49y18BGtbjTkFHcTG",
        ),
    )

    def __init__(self, auth: Auth, org_id: str, env: str = "production", **kwargs) -> None:
        super().__init__(env=env, auth=auth, **kwargs)
        tenant = get_slug_or_org_id(org_id)
        self.base_url = self.base_url.format(tenant=tenant)
        self.endpoint = RequestsEndpoint(self.base_url.rstrip("/") + "/graphql", session=self.session)

    def get_facilities(self) -> List[Facility]:
        op = Operation(Query)
        op.facilities().nodes().__fields__(*list(Facility._ContainerTypeMeta__fields.keys()))
        return (op + self.run(op)).facilities.nodes

    def create_facility(self, data: dict) -> Facility:
        op = Operation(Mutation)
        op.create_facility(input={"facility": data}).__fields__("facility")
        return self.run(op)["data"]["createFacility"]["facility"]

    def get_metric_labels(self) -> List[MetricLabel]:
        op = Operation(Query)
        op.metric_labels().__fields__(*list(MetricLabel._ContainerTypeMeta__fields.keys()))
        return self.run(op).get("data").get("metricLabels")

    def get_metric_data(self, label: str, source_id: str) -> List[MetricData]:
        op = Operation(Query)
        fields = list(MetricData._ContainerTypeMeta__fields.keys())
        fields.remove("id")
        op.metric_data(source_id=source_id, label=label).nodes().__fields__(*fields)
        return (op + self.run(op)).metric_data.nodes



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['contxt',
 'contxt.auth',
 'contxt.cli',
 'contxt.cli.commands',
 'contxt.models',
 'contxt.services',
 'contxt.utils']

package_data = \
{'': ['*']}

install_requires = \
['auth0-python>=3,<4',
 'click>=7,<9',
 'pyjwt>=2,<3',
 'pyyaml>=5,<7',
 'requests>=2,<3',
 'sgqlc>=15,<17',
 'tabulate']

extras_require = \
{'crypto': ['cryptography>=1.4']}

entry_points = \
{'console_scripts': ['contxt = contxt.__main__:cli']}

setup_kwargs = {
    'name': 'contxt-sdk',
    'version': '6.0.0',
    'description': 'Contxt SDK from ndustrial',
    'long_description': '# Contxt Python SDK\n\n[![CI](https://github.com/ndustrialio/contxt-sdk-python/workflows/CI/badge.svg)](https://github.com/ndustrialio/contxt-sdk-python/actions?query=workflow%3ACI)\n[![pypi version](https://img.shields.io/pypi/v/contxt-sdk.svg)](https://pypi.org/project/contxt-sdk/)\n![python](https://img.shields.io/badge/python-3.7+-blue.svg)\n\n## Installation\n\n```sh\npip install contxt-sdk\n```\n\n## CLI Usage\n\n```sh\ncontxt --help\n```\n\n## Documentation\n\nPlease refer to <https://ndustrialio.github.io/contxt-sdk-python>.\n\n## Contributing\n\nPlease refer to [CONTRIBUTING.md](CONTRIBUTING.md).\n',
    'author': 'ndustrial',
    'author_email': 'dev@ndustrial.io',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/ndustrialio/contxt-sdk-python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
